@@ -73,17 +73,18 @@
 #define OPT_REALTIME		(0x00000020)	/* Run with Real Time scheduling */
 #define OPT_EXTRA		(0x00000040)	/* Show extra stats */
 #define OPT_GLYPH		(0x00000080)	/* Show glyphs */
+#define OPT_COMM		(0x00000100)	/* Show comm info */
 
-#define OPT_EV_FORK		(0x00000100)	/* Fork event */
-#define OPT_EV_EXEC		(0x00000200)	/* Exec event */
-#define OPT_EV_EXIT		(0x00000400)	/* Exit event */
-#define OPT_EV_CORE		(0x00000800)	/* Coredump event */
-#define OPT_EV_COMM		(0x00001000)	/* Comm proc info event */
-#define OPT_EV_CLNE		(0x00002000)	/* Clone event */
-#define OPT_EV_PTRC		(0x00004000)	/* Ptrace event */
-#define OPT_EV_UID		(0x00008000)	/* UID event */
-#define OPT_EV_SID		(0x00010000)	/* SID event */
-#define OPT_EV_MASK		(0x0001ff00)	/* Event mask */
+#define OPT_EV_FORK		(0x00100000)	/* Fork event */
+#define OPT_EV_EXEC		(0x00200000)	/* Exec event */
+#define OPT_EV_EXIT		(0x00400000)	/* Exit event */
+#define OPT_EV_CORE		(0x00800000)	/* Coredump event */
+#define OPT_EV_COMM		(0x01000000)	/* Comm proc info event */
+#define OPT_EV_CLNE		(0x02000000)	/* Clone event */
+#define OPT_EV_PTRC		(0x04000000)	/* Ptrace event */
+#define OPT_EV_UID		(0x08000000)	/* UID event */
+#define OPT_EV_SID		(0x10000000)	/* SID event */
+#define OPT_EV_MASK		(0x1ff00000)	/* Event mask */
 #define OPT_EV_ALL		(OPT_EV_MASK)	/* All events */
 
 #define	GOT_TGID		(0x01)
@@ -286,7 +287,7 @@ static char *secs_to_str(const double secs)
 	s += 0.0005;	/* Round up */
 	fract = (s * second_scales[i].base) - (double)((int)s * second_scales[i].base);
 	(void)snprintf(buf, sizeof(buf), "%3u.%3.3u%c",
-		(int)s, (int)fract, second_scales[i].ch);
+		(unsigned int)s, (unsigned int)fract, second_scales[i].ch);
 	return buf;
 }
 
@@ -898,6 +899,34 @@ static void proc_stats_free(void)
 	}
 }
 
+/*
+ *  proc_name_clean()
+ *	clean up unwanted chars from process name
+ */
+static void proc_name_clean(char *buffer, const int len)
+{
+	char *ptr;
+
+	/*
+ 	 *  Convert '\r' and '\n' into spaces
+	 */
+	for (ptr = buffer; *ptr && (ptr < buffer + len); ptr++) {
+		if ((*ptr == '\r') || (*ptr =='\n')) 
+			*ptr = ' ';
+	}
+	/*
+	 *  OPT_CMD_SHORT option we discard anything after a space
+	 */
+	if (opt_flags & OPT_CMD_SHORT) {
+		for (ptr = buffer; *ptr && (ptr < buffer + len); ptr++) {
+			if (*ptr == ' ') {
+				*ptr = '\0';
+				break;
+			}
+		}
+	}
+}
+
 /*
  *  proc_comm()
  *	get process name from comm field
@@ -917,6 +946,8 @@ static char *proc_comm(const pid_t pid)
 	}
 	(void)close(fd);
 	buffer[ret - 1] = '\0';		/* remove trailing '\n' */
+	proc_name_clean(buffer, ret);
+
 	return strdup(buffer);
 }
 
@@ -926,11 +957,13 @@ static char *proc_comm(const pid_t pid)
  */
 static char *proc_cmdline(const pid_t pid)
 {
-	char *ptr;
 	int fd;
 	ssize_t ret;
 	char buffer[4096];
 
+	if (opt_flags & OPT_COMM)
+		return proc_comm(pid);
+
 	(void)snprintf(buffer, sizeof(buffer), "/proc/%d/cmdline", pid);
 	if ((fd = open(buffer, O_RDONLY)) < 0)
 		return proc_comm(pid);
@@ -947,6 +980,8 @@ static char *proc_cmdline(const pid_t pid)
 	 *  OPT_CMD_LONG option we get the full cmdline args
 	 */
 	if (opt_flags & OPT_CMD_LONG) {
+		char *ptr;
+
 		for (ptr = buffer; ptr < buffer + ret; ptr++) {
 			if (*ptr == '\0') {
 				if (*(ptr + 1) == '\0')
@@ -955,17 +990,8 @@ static char *proc_cmdline(const pid_t pid)
 			}
 		}
 	}
-	/*
-	 *  OPT_CMD_SHORT option we discard anything after a space
-	 */
-	if (opt_flags & OPT_CMD_SHORT) {
-		for (ptr = buffer; *ptr && (ptr < buffer + ret); ptr++) {
-			if (*ptr == ' ') {
-				*ptr = '\0';
-				break;
-			}
-		}
-	}
+
+	proc_name_clean(buffer, ret);
 
 	if (opt_flags & OPT_CMD_DIRNAME_STRIP)
 		return strdup(basename(buffer));
@@ -1622,6 +1648,7 @@ static int monitor(const int sock)
 						info1->kernel_thread ? "]" : "",
 						comm);
 					free(comm);
+					proc_info_update(pid);
 				}
 				break;
 #endif
@@ -1640,19 +1667,22 @@ static int monitor(const int sock)
 static void show_help(char *const argv[])
 {
 	(void)printf("%s, version %s\n\n", APP_NAME, VERSION);
-	(void)printf("usage: %s [-d|-D|-e|-E|-g|-h|-l|-s|-S|-q]\n", argv[0]);
-	(void)printf("-d\tstrip off directory path from process name.\n"
-	       "-D\tspecify run duration in seconds.\n"
-	       "-e\tselect which events to monitor.\n"
-	       "-E\tequivalent to -e all.\n"
-	       "-g\tshow glyphs for event types.\n"
-	       "-h\tshow this help.\n"
-	       "-l\tforce stdout line buffering.\n"
-	       "-r\trun with real time FIFO scheduler.\n"
-	       "-s\tshow short process name.\n"
-	       "-S\tshow event statistics at end of the run.\n"
-	       "-q\trun quietly and enable -S option.\n"
-	       "-x\tshow extra process information.\n");
+	(void)printf("usage: %s [-c|-d|-D|-e|-E|-g|-h|-l|-s|-S|-q|-x|-X]\n", argv[0]);
+	(void)printf(
+		"-c\tuse task comm field for process name.\n"
+		"-d\tstrip off directory path from process name.\n"
+		"-D\tspecify run duration in seconds.\n"
+		"-e\tselect which events to monitor.\n"
+		"-E\tequivalent to -e all.\n"
+		"-g\tshow glyphs for event types.\n"
+		"-h\tshow this help.\n"
+		"-l\tforce stdout line buffering.\n"
+		"-r\trun with real time FIFO scheduler.\n"
+		"-s\tshow short process name.\n"
+		"-S\tshow event statistics at end of the run.\n"
+		"-q\trun quietly and enable -S option.\n"
+		"-x\tshow extra process information.\n"
+		"-X\tequivalent to -EgrSx.\n");
 }
 
 /*
@@ -1691,10 +1721,13 @@ int main(int argc, char * const argv[])
 	struct sigaction new_action;
 
 	for (;;) {
-		const int c = getopt(argc, argv, "dD:e:EghlrsSqx");
+		const int c = getopt(argc, argv, "cdD:e:EghlrsSqxX");
 		if (c == -1)
 			break;
 		switch (c) {
+		case 'c':
+			opt_flags |= OPT_COMM;
+			break;
 		case 'd':
 			opt_flags |= OPT_CMD_DIRNAME_STRIP;
 			break;
@@ -1740,6 +1773,9 @@ int main(int argc, char * const argv[])
 		case 'x':
 			opt_flags |= OPT_EXTRA;
 			break;
+		case 'X':
+			opt_flags |= (OPT_EV_ALL | OPT_GLYPH | OPT_STATS | OPT_EXTRA | OPT_REALTIME);
+			break;
 		default:
 			show_help(argv);
 			exit(EXIT_FAILURE);
